<?php


/**
 * This file is part of the OpenPNE package.
 * (c) OpenPNE Project (http://www.openpne.jp/)
 *
 * For the full copyright and license information, please view the LICENSE
 * file and the NOTICE file that were distributed with this source code.
 */

/**
 * activeTransition actions.
 *
 * @package    OpenPNE
 * @subpackage activeTransition
 * @author     Your name here
 * @version    SVN: $Id: actions.class.php 9301 2008-05-27 01:08:46Z dwhittle $
 */
class activeTransitionActions extends sfActions {
    /**
     * Executes index action
     *
     * @param sfWebRequest $request A request object
     */
    public function executeIndex(sfWebRequest $request) {
        $this->forward('activeTransition', 'menu');
    }

    /**
     * Executes menu action
     *
     * @param sfWebRequest $request A request object
     */
    public function executeMenu(sfWebRequest $request) {
        $this->form = new ActiveTransitionForm();

        $this->pager = new sfDoctrinePager('ActiveTransition', sfConfig :: get('app_max_logs_on_active_transition'));
        $this->pager->setQuery(Doctrine_Query :: create()->from('ActiveTransition a')->orderBy('id DESC'));
        $this->pager->setPage($request->getParameter('page', 1));
        $this->pager->init();
    }

    /**
     * Executes introduction action
     *
     * @param sfWebRequest $request A request object
     */
    public function executeIntroduction(sfWebRequest $request) {
    }

    /**
     * Executes terms of service action
     *
     * @param sfWebRequest $request A request object
     */
    public function executeTos(sfWebRequest $request) {
    }

    /**
     * Executes create action
     *
     * @param sfWebRequest $request A request object
     */
    public function executeCreate(sfWebRequest $request) {
        $this->forward404Unless($request->isMethod(sfRequest::POST));
        $this->form = new ActiveTransitionForm();
        // フォームのvalidationを行う．
        $this->processForm($request, $this->form);
        
        // フォームが正当でなかった場合
        // executeMenuに進むための処理
        $this->pager = new sfDoctrinePager('ActiveTransition', sfConfig::get('app_max_logs_on_active_transition'));
        $this->pager->setQuery(Doctrine_Query :: create()->from('ActiveTransition a')->orderBy('id DESC'));
        $this->pager->setPage($request->getParameter('page', 1));
        $this->pager->init();
        $this->setTemplate('menu');
    }

    /**
     * 状態更新用
     */
    public function executeRefresh(sfWebRequest $request) {
        $this->forward404Unless($request->isMethod(sfRequest::POST));
        $at = ActiveTransitionTable::getUnderConstruction();
        if ($at) {
            require_once "HTTP/Client.php";
            $client = & new HTTP_Client();
            $host = sfConfig::get('app_checking_url_on_active_transition');
            $params = array('hash' => ActiveTransitionHashTable::getHash());
            $client->post('https://' . $host, $params);
            $response = $client->currentResponse();
            if ($response === null) {
                $client->post('http://' . $host, $params);
                $response = $client->currentResponse();
            }
            if ($response === null) { // 接続エラー
                $at->status = 31;
                $at->save();
            } else if ($response['code'] === 404) { // 404 Not Found
                $at->status = 32;
                $at->save();
            } else {
                switch ($response['body']) {
                  case 'under construction': // 作成中
                    $at->status = 30;
                    $at->save();
                    break;
                  case 'invalid request': // 不正なリクエスト
                    $at->status = 33;
                    $at->save();
                    break;
                  case 'unknown applicant': // 申請履歴が見つからない
                    $at->status = 40;
                    $at->save();
                    break;
                  case 'construction failed': // 作成失敗
                    $at->status = 50;
                    $at->save();
                    break;
                  case 'completed': // 作成完了
                    $at->status = 60;
                    $at->save();
                    break;
                  default: // 規定外のレスポンス
                    $at->status = 39;
                    $at->save();
                    break;
                }
            }
        }
        $this->forward('activeTransition', 'menu');
    }

    /**
     * 実際にデータの送信を行う部分
     */
    public function executeLoop(sfWebRequest $request) {
        $this->forward404Unless($request->isMethod(sfRequest::POST));
        // 受け取ったパラメータの処理
        $loopNumber = $request->getParameter('loopNumber'); // 累計ループ番号
        $tableId = $request->getParameter('tableId');       // テーブル番号
        $rowId = $request->getParameter('rowId');           // 取得開始位置
        $email = $request->getParameter('email');           // 送信先メールアドレス
        $limit = 20000;                                     // 一度に送信する数
        
        $tableNameArray = array_keys(ActiveTransition::$seekingColumns);
        $tableSize = count($tableNameArray);
        
        // zipアーカイブの準備
        $tmpfname = tempnam("/tmp", "at");
        $zip = new ZipArchive;
        if ($zip->open($tmpfname, ZipArchive::OVERWRITE) !== TRUE) {
            return sfView :: ERROR;
        }
        
        if ($tableId >= $tableSize) {
            // 全てのテーブルを送信完了
            $this->data_query = '-1';
            $loopNumber = -1;
        } else {
            $this->createCompactZipSNSData($zip, $tableId, $rowId, $limit);
            if ($limit > 0) {
                // 全てのデータを送信完了
                $this->data_query = '-1';
                $loopNumber = -1;
            } else {
                $this->data_query = ($loopNumber+1) . ":" . $tableId . ":" . $rowId;
            }
        }
        $zip->close();
        
        // データの送信
        $response = ActiveTransition::postData(
            sfConfig::get('app_posting_url_on_active_transition'),
            $tmpfname, $email, $loopNumber);
        
        if ($response === null) { // サーバ接続エラー
            ActiveTransitionTable::updateStatus(11);
            $this->data_query .= ':' . ActiveTransition::$statuses[11];
        } else if ($response['code'] === 404) { // 404 Not Found
            ActiveTransitionTable::updateStatus(12);
            $this->data_query .= ':' . ActiveTransition::$statuses[12];
        } else {
            switch ($response['body']) {
              case 'succeeded uploading': // 送信成功
                if ($this->data_query == '-1') { // 送信完了
                    ActiveTransitionTable::updateStatus(20);
                    $this->data_query .= ':' . ActiveTransition::$statuses[20];
                } else {
                    $this->data_query .= ':' . ActiveTransition::$statuses[20];
                }
                break;
              case 'invalid request': // 不正なリクエスト
                ActiveTransitionTable::updateStatus(13);
                $this->data_query .= ':' . ActiveTransition::$statuses[13];
                break;
              case 'unknown identity': // 識別IDが見つからなかった
                ActiveTransitionTable::updateStatus(14);
                $this->data_query .= ':' . ActiveTransition::$statuses[14];
                break;
              case 'internal server error': // サーバ内部エラー
                ActiveTransitionTable::updateStatus(15);
                $this->data_query .= ':' . ActiveTransition::$statuses[15];
                break;
              case 'still on progress': // 前回の送信データが処理中
                ActiveTransitionTable::updateStatus(18);
                $this->data_query .= ':' . ActiveTransition::$statuses[18];
                break;
              default: // 規定外のレスポンス
                ActiveTransitionTable::updateStatus(19);
                $this->data_query .= ':' . ActiveTransition::$statuses[19];
                break;
            }
        }
    }

    /**
     * 識別IDの発行，最終確認まで．
     */
    public function executeSend(sfWebRequest $request) {
        $this->forward404Unless($request->isMethod(sfRequest::POST));
        // 入力されたデータの取得
        $formData = $request->getParameter('active_transition');
        $this->email = $formData['email'];
        $this->succeed = false;

        // 識別用IDの管理
        if (ActiveTransitionHashTable::getHash()) {
            $this->result = ActiveTransitionHash::checkHash();
            // 識別IDが既に割り当てられている場合
            $response = ActiveTransitionHash::checkHash();
            if ($response === null) { // サーバ接続エラー
                ActiveTransitionTable::updateStatus(1);
                $this->result = ActiveTransition::$statuses[1];
            } else if ($response['code'] === 404) { // 404 Not Found
                ActiveTransitionTable::updateStatus(2);
                $this->result = ActiveTransition::$statuses[2];
            } else if ($response['body'] === 'ok') {
                ActiveTransitionTable::updateStatus(10);
                $this->succeed = true;
            } else { // 規定外のメッセージ
                ActiveTransitionTable::updateStatus(9);
                $this->result = ActiveTransition::$statuses[9];
            }
        } else {
            // 新たに識別IDを発行する場合
            $response = ActiveTransitionHash::receiveNewHash();
            if ($response === null) { // サーバ接続エラー
                ActiveTransitionTable::updateStatus(1);
                $this->result = ActiveTransition::$statuses[1];
            } else if ($response['code'] === 404) { // 404 Not Found
                ActiveTransitionTable::updateStatus(2);
                $this->result = ActiveTransition::$statuses[2];
            } else {
                // メッセージとIDを含んだ配列
                $messageAndId = explode(':', $response['body']);
                if ($messageAndId[0] === 'your_id') { // メッセージ部分が正当
                    // 識別文字列を設定
                    ActiveTransitionHashTable::setHash($messageAndId[1]);
                    ActiveTransitionTable::updateStatus(10);
                    $this->succeed = true;
                } else { // 規定外のメッセージ
                    ActiveTransitionTable::updateStatus(9);
                    $this->result = ActiveTransition::$statuses[9];
                }
            }
        }
    }

    /**
     * 連続送信用のZIPを作成．
     */
    private function createCompactZipSNSData(&$zip, &$tableId, &$rowId, &$limit) {
        $tableNameArray = array_keys(ActiveTransition::$seekingColumns); // 利用するテーブル名の一覧
        $tableSize = count($tableNameArray); // テーブルの数
        
        while ($tableId < $tableSize && $limit > 0) {
            $tableName = $tableNameArray[$tableId];
            $fileName = $tableName . ".csv";
            
            // データの取得
            $query = ActiveTransition::createQuery($tableName, $rowId, $limit);
            $rs = ActiveTransition::retrieveData($query);
            $size = count($rs);
            
            // 結果をzipに含める
            $csvData = ActiveTransition::getCsvFromResultSet($tableName, $rs);
            $zip->addFromString($fileName, $csvData);
            
            // 次のループの準備
            $limit -= $size;
            $rowId += $size;
            if ($limit > 0) { // まだ送信上限に達していない場合は次のテーブルへ
                $tableId++;
                $rowId = 0;
            }
        }
    }

    /**
     * menuSuccess.phpで入力されたフォームデータの正当性チェックを行う．
     */
    public function processForm(sfWebRequest $request, sfForm $form) {
        $form->bind($request->getParameter($form->getName()), $request->getFiles($form->getName()));
        if ($form->isValid()) { // フォーム内容が正しければログを記録し，executeSendへforward
            $form->save();
            $this->forward('activeTransition', 'send');
        }
    }
}
